"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.contentPathFromHex = exports.getFilePathByModeInCafs = exports.modeIsExecutable = void 0;
const path = require("path");
const ssri = require("ssri");
exports.modeIsExecutable = (mode) => (mode & 0o111) === 0o111;
function getFilePathByModeInCafs(cafsDir, integrity, mode) {
    const fileType = exports.modeIsExecutable(mode) ? 'exec' : 'nonexec';
    return path.join(cafsDir, contentPathFromIntegrity(integrity, fileType));
}
exports.getFilePathByModeInCafs = getFilePathByModeInCafs;
function getFilePathInCafs(cafsDir, integrity, fileType) {
    return path.join(cafsDir, contentPathFromIntegrity(integrity, fileType));
}
exports.default = getFilePathInCafs;
function contentPathFromIntegrity(integrity, fileType) {
    const sri = ssri.parse(integrity, { single: true });
    return contentPathFromHex(fileType, sri.hexDigest());
}
function contentPathFromHex(fileType, hex) {
    const p = path.join(hex.slice(0, 2), hex.slice(2));
    switch (fileType) {
        case 'exec':
            return `${p}-exec`;
        case 'nonexec':
            return p;
        case 'index':
            return `${p}-index.json`;
    }
}
exports.contentPathFromHex = contentPathFromHex;
//# sourceMappingURL=getFilePathInCafs.js.map